function xrec = PPXA_Gaussian_Hybrid_TV_L1(x,M,rm,niter,z,T,h,L,alpha,nomImage,gamma,lambda,coef,alphap,betap,pp,mup,alpham,betam,pm,mum,minx,maxx)


% INITIALIZATION
%----------------

Nx=length(x);           % Image size Nx^2 
mu=2;                   % Frame constante


%Number of functional 
%--------------------
flou = 1;       %Data fidelity term
reg = 1;        %Frame regularization
cc = 1;         %Convex constraint
tv=4;           %Total variation [Combettes&Pesquet2008]

N1=flou +reg +cc +tv;
fprintf('\n Proximity operators number \t: %d\n',N1);
fprintf('----------------------------\n');
fprintf('Proximity operators related to data fidelity term \t: %d\n',flou);
fprintf('Proximity operators related to the frame regularization term \t: %d\n',reg);
fprintf('Projection onto the convex constraint \t: %d\n',cc);
fprintf('Proximity operators related to TV \t: %d\n',tv);



%Algorithm initialization
%------------------------
xrec=z;
[wzp,wzm,H1x,H2x,HH1x,HH2x,Gx,Ghx]=dualtreeM2D_freq(z,M,rm);

Ml=length(wzp{1});
Mc=length(wzp{1}{1});
wzp_vect = cell2vect(wzp,rm,Ml,Mc);
wzm_vect = cell2vect(wzm,rm,Ml,Mc);

wynip = ones(1,1)*wzp_vect;
wynim = ones(1,1)*wzm_vect;

wpnip = ones(1,1)*zeros(size(wzp_vect));
wpnim = ones(1,1)*zeros(size(wzm_vect));

zrec = idualtreeM2D_freq_deblur(wzp,wzm,rm,H1x,H2x,HH1x,HH2x,Gx,Ghx);
zni = ones(N1-1,1)*zrec(:)';

[FFstar_wzp,FFstar_wzm,H1x,H2x,HH1x,HH2x,Gx,Ghx]=dualtreeM2D_freq(zrec,M,rm);
FFstar_wzp_vect = cell2vect(FFstar_wzp,rm,Ml,Mc);
FFstar_wzm_vect = cell2vect(FFstar_wzm,rm,Ml,Mc);

wynip_ortho = ones(N1-1,1)*(wzp_vect - 1/mu * FFstar_wzp_vect);
wynim_ortho = ones(N1-1,1)*(wzm_vect - 1/mu * FFstar_wzm_vect);

qni = ones(N1-1,1)*zeros(size(wzp_vect));


%Weighted constants initialization
%---------------------------------
wi = 1/N1*ones(N1,1);
wi(1) = 1/4;
wi(end)=1/4;
wi(flou+1)=1/4;
wi(flou+2:flou+5)=1/(4*4);


if (sum(wi)>1.0001 || sum(wi)<0.9999 ) 
    fprintf('ERREUR: The wi''s is non equal to 1. The value is %3.7f\n\n',sum(wi));
end

% wxn initialization
%-------------------
wxnp_temp = wynip;
wxnm_temp = wynim;
wxnp = sum(wxnp_temp,1);
wxnm = sum(wxnm_temp,1);



% Criterion initialization
%-------------------------
snr = zeros(1,niter);
distQUAD=zeros(1,niter);
sumcoef=zeros(1,niter);
tv=zeros(1,niter);
dc=zeros(1,niter);
critere=zeros(1,niter);
time = zeros(1,niter+1);

fprintf('\n Start PPXA \n'); 
fprintf('------------- \n');

%Algorithm
%---------
time(1)=0;
for n=1:niter
   
    wxnpcell = vect2cell(wxnp,rm,Ml,Mc,Nx);
    wxnmcell = vect2cell(wxnm,rm,Ml,Mc,Nx);
   
    sumcoef(n)=sumcoefadapModif_ME2(wxnpcell,alphap,betap,pp,1,mup)+sumcoefadapModif_ME2(wxnmcell,alpham,betam,pm,1,mum);
    distQUAD(n) = quad(z,xrec,T,alpha) ;
    snr(n) = snrdeblur(x,xrec);
    tv(n) = Variation_totale(xrec) ;
    dc(n) = distance_convexe(xrec,minx,maxx);
    critere(n)= distQUAD(n) + coef(1)*sumcoef(n) +coef(2)*tv(n);
    if(mod(n,10)==0) 
        fprintf('%d : dc = %3.2f  |  critere = %3.2f |  snr = %3.2f \n',n,dc(n),critere(n), snr(n));
    end
    
    % Proximity operators computation
    %--------------------------------
    
    tic;  
    
    ind=1;
    %relative to data fidelity term
    [qni(ind,:)]=1/mu*prox_quad(zni(ind,:),Nx,z,T,alpha,wi(ind,1),mu*gamma);

    % relative to convex constraint projection
    ind = flou + 1;
    [qni(ind,:)]=1/mu*projectionDTT(zni(ind,:),minx,maxx);
 
    % relative to total variation  
   [qni(flou+2,:)] = 1/mu*prox_tv_partiel(zni(flou+2,:),Nx,0,0,wi(flou+2,:),gamma,coef(2),mu);
   [qni(flou+3,:)] = 1/mu*prox_tv_partiel(zni(flou+3,:),Nx,0,1,wi(flou+3,:),gamma,coef(2),mu);
   [qni(flou+4,:)] = 1/mu*prox_tv_partiel(zni(flou+4,:),Nx,1,0,wi(flou+4,:),gamma,coef(2),mu);
   [qni(flou+5,:)] = 1/mu*prox_tv_partiel(zni(flou+5,:),Nx,1,1,wi(flou+5,:),gamma,coef(2),mu);
 
    % relative to frame regularization
    ind = N1;  
    wynip_regcell = vect2cell(wynip(1,:),rm,Ml,Mc,Nx);
    wynim_regcell = vect2cell(wynim(1,:),rm,Ml,Mc,Nx);
     for j=1:rm,
         for ml=1:Ml,
             for mc=1:Mc,
                 if (j==rm && ml==1 && mc==1),
                    % Approximation frame coefficients
                    omegap(1)=coef(1)*gamma*alphap{j}{ml}{mc}/wi(ind,1);
                    omegap(2)=coef(1)*gamma*betap{j}{ml}{mc}/wi(ind,1);
                    wynip_regcell{j}{ml}{mc}= mup + proxabspowerp(wynip_regcell{j}{ml}{mc}-mup,pp{j}{ml}{mc},omegap);
                    
                    omegam(1)=coef(1)*gamma*alpham{j}{ml}{mc}/wi(ind,1);
                    omegam(2)=coef(1)*gamma*betam{j}{ml}{mc}/wi(ind,1);
                    wynim_regcell{j}{ml}{mc}= mum + proxabspowerp(wynim_regcell{j}{ml}{mc}-mum,pm{j}{ml}{mc},omegam);
                    
                  elseif ~isempty(wynim_regcell{j}{ml}{mc})
                    % Details frame coefficients                     
                     omegap(1)=coef(1)*gamma*alphap{j}{ml}{mc}/wi(ind,1);
                     omegap(2)=coef(1)*gamma*betap{j}{ml}{mc}/wi(ind,1);
                     wynip_regcell{j}{ml}{mc}= proxabspowerp(wynip_regcell{j}{ml}{mc},pp{j}{ml}{mc},omegap);
                     
                     omegam(1)=coef(1)*gamma*alpham{j}{ml}{mc}/wi(ind,1);
                     omegam(2)=coef(1)*gamma*betam{j}{ml}{mc}/wi(ind,1);
                     wynim_regcell{j}{ml}{mc}= proxabspowerp(wynim_regcell{j}{ml}{mc},pm{j}{ml}{mc},omegam);

                 end
              end
          end
     end
    wpnip(1,:) = cell2vect(wynip_regcell,rm,Ml,Mc);
    wpnim(1,:) = cell2vect(wynim_regcell,rm,Ml,Mc); 
    
    
    % Weighted sum
    %-------------    
    pn_temp1 = (wi(1:N1-1)*ones(1,length(qni))).*qni;
    pn_temp1_img = reshape(sum(pn_temp1,1),Nx,Nx);
    [wpnp1_cell,wpnm1_cell,H1x,H2x,HH1x,HH2x,Gx,Ghx]=dualtreeM2D_freq(pn_temp1_img,M,rm);
    wpnp1 = cell2vect(wpnp1_cell,rm,Ml,Mc);
    wpnm1 = cell2vect(wpnm1_cell,rm,Ml,Mc);  
        
    wpnp_temp2 = (wi(1:N1-1)*ones(1,length(wynip_ortho))).*wynip_ortho;
    wpnp2 = sum(wpnp_temp2,1);
    wpnm_temp2 = (wi(1:N1-1)*ones(1,length(wynim_ortho))).*wynim_ortho;
    wpnm2 = sum(wpnm_temp2,1);   

    wpnp_temp3 = (wi(N1)*ones(1,length(wpnip(1,:)))).*wpnip(1,:);
    wpnp3 = sum(wpnp_temp3,1);
    wpnm_temp3 = (wi(N1)*ones(1,length(wpnim(1,:)))).*wpnim(1,:);
    wpnm3 = sum(wpnm_temp3,1);    

    wpnp = wpnp1+ wpnp2+ wpnp3;  
    wpnm = wpnm1+ wpnm2+ wpnm3;  
    
    
    %rl and rl tilde computation
    %---------------------------
    wrnp = 2*wpnp - wxnp;
    wrnm = 2*wpnm - wxnm; 
    
    wrnp_cell = vect2cell(wrnp,rm,Ml,Mc,Nx);
    wrnm_cell = vect2cell(wrnm,rm,Ml,Mc,Nx);
    rn = idualtreeM2D_freq_deblur(wrnp_cell,wrnm_cell,rm,H1x,H2x,HH1x,HH2x,Gx,Ghx);
    rnl = rn(:)';
    
    [Frlp_cell,Frlm_cell,H1x,H2x,HH1x,HH2x,Gx,Ghx]=dualtreeM2D_freq(rn,M,rm);
    Frlp = cell2vect(Frlp_cell,rm,Ml,Mc);
    Frlm = cell2vect(Frlm_cell,rm,Ml,Mc);    
    
    wrnp_ortho = wrnp - 1/mu*Frlp;  
    wrnm_ortho = wrnm - 1/mu*Frlm;   
    
        
    % yn update
    %----------    
    wynip_ortho = wynip_ortho + lambda*(ones(N1-1,1)*wrnp_ortho - wynip_ortho);
    wynim_ortho = wynim_ortho + lambda*(ones(N1-1,1)*wrnm_ortho - wynim_ortho);
    
    zni = zni + lambda*(ones(N1-1,1)*rnl - mu*qni);
        
    wynip(1,:) = wynip(1,:) + lambda*(ones(1,1)*wrnp - wpnip(1,:));
    wynim(1,:) = wynim(1,:) + lambda*(ones(1,1)*wrnm - wpnim(1,:));

   
    
    % xn update
    %----------
    wxnp = wxnp + lambda * (wpnp - wxnp);
    wxnm = wxnm + lambda * (wpnm - wxnm); 
        

    time(n+1) = time(n)+ toc;  
    
    
    %Transform domain to image domain
    %--------------------------------
    wprec = vect2cell(wxnp,rm,Ml,Mc,Nx);
    wmrec = vect2cell(wxnm,rm,Ml,Mc,Nx);
    xrec = idualtreeM2D_freq_deblur(wprec,wmrec,rm,H1x,H2x,HH1x,HH2x,Gx,Ghx);
      
end
fprintf('\n\n PPXA Hybrid l1+tv:\n');
fprintf('----------\n');
fprintf('time = %5.1f  \n',time(n+1));
fprintf('snr = %3.2f\n',snr(n));
fprintf('ssim = %3.3f\n',ssim(x,xrec));
fprintf('distance to C = %3.2f  \n',dc(n));

figure(1)
subplot(131)
imagesc(x);
axis image off
subplot(132)
imagesc(z);
axis image off
subplot(133)
imagesc(xrec);
axis image off
colormap(gray)
figure(2)
subplot(311)
plot(snr)
title('SNR')
subplot(312)
plot(critere)
title('Objective function')
subplot(313)
plot(dc)
title('Distance to C')
%%%%%%%%%%%%%%%%% SAVE DATA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tit=['PPXA-GaussianDeblur-Hybrid-l1DTT-TV-',nomImage,'-niter',int2str(niter),'alpha',num2str(alpha),'L',int2str(L),'coef_reg',num2str(coef(1)),'coef_tv',num2str(coef(2)),'gamma',num2str(gamma),'lambda',num2str(lambda),'-rm=',int2str(rm),'.mat'];
save(tit,'x','z','xrec','snr','critere','time','distQUAD','sumcoef','tv','N1','dc','wprec','wmrec');
